#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Группа программ: "Использование Unicode в Perl"
# Программа: обеспечение вывода сообщения об ошибке открытия файла 
#            на РУССКОМ языке (без искажения кодировки)
# Автор:     Моргунов Е. П.
# Дата:      18.12.2013
# Версия:    0.1
# -----------------------------------------------------------------------------

# эта строка означает строгую проверку синтаксиса исходного текста программы
use strict;

# этот модуль нужен для функции decode()
use Encode;

# эта директива означает, что ИСХОДНЫЙ ТЕКСТ скрипта представлен в Unicode
use utf8;

# Нназначим вывод в кодировке UTF-8 для устройства STDERR (стандартное 
# устройство вывода сообщений об ошибках), т. к. функция die() выводит
# сообщение на STDERR (см. perldoc -f die).
# Попробуйте закомментировать эту команду. Будет выведено предупреждение
# "Wide character in die at ..."
binmode STDERR, ':encoding(UTF-8)'; # можно сделать и так (хотя эти два метода 
                                    # НЕ ТОЖДЕСТВЕННЫ): binmode STDOUT, ':utf8';
my $file = "abcd.txt";

# попытаемся открыть несуществующий файл, чтобы заведомо вызвать вывод сообщения
# об ошибке

# Приводим два варианта команды открытия файла (несуществующего). 
# Для экспериментирования нужно закомментировать один из вариантов,
# оставив другой. Сравните, что выводит программа на экран в том и в другом
# случае.

# Первый вариант команды.
# Текст, выводимый вместо встроенной переменной $!, будет выведен 
# в неправильной кодировке.
open( READ, "<", "$file" ) || die "Не могу открыть файл $file: $!";

# Второй вариант команды.
# Функция decode() преобразует сообщение, соответствующее переменной $!,
# в правильную кодировку.
#open( READ, "<", "$file" ) || die "Не могу открыть файл $file: " .
#  decode( 'UTF-8', $! );

while( <READ> )
{
  print;
}

close( READ );

exit 0;
