#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Группа программ: "Использование Unicode в Perl"
# Программа: вывод текстовых файлов, содержащих Unicode
# Автор:     Моргунов Е. П.
# Дата:      18.12.2013
# Версия:    0.1
# -----------------------------------------------------------------------------

# эта строка означает строгую проверку синтаксиса исходного текста программы
use strict;

# этот модуль нужен для функции decode()
use Encode;

# эта директива означает, что ИСХОДНЫЙ ТЕКСТ скрипта представлен в Unicode
use utf8;

# Нназначим вывод в кодировке UTF-8 для устройства STDERR (стандартное 
# устройство вывода сообщений об ошибках), т. к. функция die() выводит
# сообщение на STDERR (см. perldoc -f die).
binmode STDERR, ':encoding(UTF-8)'; # можно сделать и так (хотя эти два метода 
                                    # НЕ ТОЖДЕСТВЕННЫ): binmode STDERR, ':utf8';

# При выводе данных на стандартный вывод также назначаем кодировку UTF-8. Если
# эту строку закомментировать, то будет выведено предупреждение "Wide character
# in print ..."
binmode STDOUT, ':encoding(UTF-8)'; # можно сделать и так (хотя эти два метода 
                                    # НЕ ТОЖДЕСТВЕННЫ): binmode STDOUT, ':utf8';
my $file = "text.txt";

# откроем файл, в котором текст представлен в кодировке Unicode
open( READ, "<", "$file" ) || die "Не могу открыть файл $file: " .
  decode( 'UTF-8', $! );

# будем выводить строки файла и их длину, которая в данной ситуации будет 
# длиной строки в БАЙТАХ, а не в СИМВОЛАХ (каждый символ русского алфавита 
# кодируется двумя байтами)
while ( <READ> )
{
  chomp;
  print decode( 'UTF-8', $_ ) . " (" . length( $_ ) . " байтов, " . 
        length( decode( 'UTF-8', $_ ) ) . " символов)\n";

  # Второй вариант команды.
  # Закомментируйте первый вариант и выполните программу снова. Что изменилось?
  #print $_ . " (" . length( $_ ) . " байтов, " . 
  #      length( decode( 'UTF-8', $_ ) ) . " символов)\n";
}

close( READ );

print "-" x 50, "\n";

# ----------------------------------------------

# снова откроем файл, в котором текст представлен в кодировке Unicode, 
# но теперь явно укажем кодировку символов
open( READ, "<:encoding(UTF-8)", "$file" ) || 
  die "Не могу открыть файл $file: $!";

# будем выводить строки файла и их длину, которая в данной ситуации будет 
# длиной строки в СИМВОЛАХ
while ( <READ> )
{
  chomp;
  print $_ . " (" . length( $_ ). ")\n";
}

close( READ );

exit 0;
