#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Группа программ: "Использование Unicode в Perl"
# Программа: работа с функциями обработки строк (русскоязычные символы)
# Автор:     Моргунов Е. П.
# Дата:      18.12.2013
# Версия:    0.1
# -----------------------------------------------------------------------------

# эта строка означает строгую проверку синтаксиса исходного текста программы
use strict;

use utf8;
use Unicode::Collate;

my $file = "text.txt";

# при выводе данных на стандартный вывод также предполагаем кодировку UTF-8
binmode STDOUT, ':utf8';
binmode STDERR, ':utf8';

my @letters = ();
my $abc = "А Б В Г Д Е Ё Ж З И Й К Л М Н О П Р С Т У Ф Х Ц Ч Ш Щ Ъ Ы Ь Э Ю Я";

@letters = split( /\s+/, $abc );

# Сортировка на основе значений кодовых точек. Найдите букву "Ё".
foreach my $letter ( sort( @letters ) )
{
  print "$letter ";
}

print "\n";

# Правильная сортировка. Найдите букву "Ё".
foreach my $letter ( Unicode::Collate->new->sort( @letters ) )
{
  print "$letter ";
}

print "\n\n";

$abc = "а б в г д е ё ж з и й к л м н о п р с т у ф х ц ч ш щ ъ ы ь э ю я";

@letters = split( /\s+/, $abc );

# Сортировка на основе значений кодовых точек. Найдите букву "ё".
foreach my $letter ( sort( @letters ) )
{
  print "$letter ";
}

print "\n";

# Правильная сортировка. Найдите букву "ё".
foreach my $letter ( Unicode::Collate->new->sort( @letters ) )
{
  print "$letter ";
}

print "\n\n";


# выражение "<:encoding(UTF-8)" пишется БЕЗ пробелов
# оно означает, что файл рассматривается, как представленный в кодировке UTF-8
open( READ, "<:encoding(UTF-8)", "$file" ) || 
  die "Не могу открыть файл $file: $!";

# Попробуйте этот способ открытия файла. Что выводится на экран?
#open( READ, "<", "$file" ) || 
#  die "Не могу открыть файл $file: $!";

my( $begin, $end );

while ( my $str = <READ> )
{
  chomp $str;

  # сначала выведем строку из файла в оригинальном виде
  print "$str\n";

  # разделим строку из файла на отдельные буквы
  @letters = split( //, $str );

  print "Массив содержит " . scalar @letters . " элемента(ов)\n";
  
  # выведем все элементы массива (отдельные буквы слова)
  foreach my $letter ( @letters )
  {
    print "$letter ";
  }
  print "\n";
  
  # перевод строки в верхний и нижний регистры
  print "Верхний регистр: " . uc( $str ) . "\n";
  print "Нижний регистр: " . lc( $str ) . "\n";

  # регулярное выражение позволяет записать в первую переменную -- три первых
  # символа строки, а во вторую переменную -- все остальные символы
  ( $begin, $end ) = $str =~ /^(...)(.*)$/;

  print "Начало: $begin\n";
  print "Конец: $end\n";

  # отчеркнем все операции, выполненные с текущей строкой файла, строкой дефисов
  print '-' x 50 . "\n";
}

close( READ );

exit( 0 );
