#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Программа: вычисление числа дней, оставшихся до дня рождения
# Автор: Моргунов Е. П.
# Дата: 3.10.2014
# -----------------------------------------------------------------------------

# Название программы -- аббревиатура от "how long till birthday".

# Для выполнения программы нужно передать ей в качестве параметра дату,
# например: 
# ./hltbd.pl 26-06-1967

use strict;

my $birthday = $ARGV[ 0 ];

my $today_day;       # сегодняшний день
my $today_month;     # сегодняшний месяц
my $today_year;      # сегодняшний год

my $birthday_day;    # день(именно день) рождения
my $birthday_month;  # месяц рождения
my $birthday_year;   # год рождения (пока не используется)

my $i;
my @days_in_months = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );

# введем 0 для фиктивного "нулевого" месяца, чтобы потом мы могли
# обращаться к элементам этого массива по номеру месяца, не уменьшая его на 1
my @days_in_two_years = ( 0 );
my $days_till_birthday = 0;

# получим значения всех переменных
( $today_day, $today_month, $today_year ) = ( localtime() )[ 3..5 ];
$today_month += 1;   # см. описание функции localtime()
$today_year += 1900; # см. описание функции localtime()

print "Сегодня $today_day-$today_month-$today_year\n";

( $birthday_day, $birthday_month, $birthday_year ) = 
  $birthday =~ m#^(\d{1,2})[-\./](\d{1,2})[-\./](\d{4})$#;

# DEBUG
# print "$birthday_day, $birthday_month, $birthday_year\n";
# print "$today_day, $today_month, $today_year\n";

# создадим массив, содержащий количества дней в каждом месяце для ДВУХ ЛЕТ
# (текущего и будущего)
push( @days_in_two_years, @days_in_months, @days_in_months );

#DEBUG
# print join( ',', @days_in_two_years ) . "\n";

# если год високосный, нужно изменить число дней в феврале
# (делаем эту проверку для текущего и для будущего годов)
$days_in_two_years[ 2 ] = 29 if ( is_leap_year( $today_year ) );
$days_in_two_years[ 14 ] = 29 if ( is_leap_year( $today_year + 1 ) );

# день рождения сегодня
if ( $birthday_month == $today_month && $birthday_day == $today_day )
{
  $days_till_birthday = 0;
}

# если (месяц + день) рождения БОЛЬШЕ текущего (месяц + день),
# значит день рождения будет в этом году
else
{
  if ( ( $birthday_month > $today_month ) ||
          ( $birthday_month == $today_month && $birthday_day > $today_day )
        )
  {
    ;  # не делаем ничего
  }
  # если (месяц + день) рождения МЕНЬШЕ текущего (месяц + день),
  # значит день рождения будет на будущий год, т.к. в этом году он уже был
  else
  {
    # увеличим номер месяца рождения на 12, т.к. это уже БУДУЩИЙ год
    # (а в нашем массиве, содержащем количества дней в месяцах, содержатся
    # эти количества для ДВУХ ЛЕТ)
    $birthday_month += 12;
  }

  # сначала получим число дней в ПОЛНЫХ месяцах, начиная с месяца, к которому
  # принадлежит текущая дата, и заканчивая месяцем, к которому принадлежит дата
  # рождения
  for ( $i = $today_month; $i <= $birthday_month; $i++ )
  {
    $days_till_birthday += $days_in_two_years[ $i ];
  }

  # теперь мы должны вычесть лишние дни, а именно...
   
  # число дней, уже прошедших с начала текущего месяца
  $days_till_birthday = $days_till_birthday - $today_day;
  
  # число дней, оставшихся с дня (именно дня) рождения до конца
  # месяца рождения
  $days_till_birthday = $days_till_birthday - 
    ( $days_in_two_years[ $birthday_month ] - $birthday_day );
}

if ( $days_till_birthday == 0 )
{
  print "День рождения сегодня\n";
}
else
{
  print "До дня рождения осталось $days_till_birthday день (дня, дней)\n";
}

exit( 0 );


sub is_leap_year
{
  my $year = shift;
  
  my $leap = 0;
  
  # упрощенный алгоритм!!!
  if ( $year % 4 == 0 )
  {
    $leap = 1;
  }

  return $leap;
}
