#!/usr/bin/perl -w

# Программа: выполнение арифметических операций с целыми числами "в столбик".
# Автор:     Моргунов Е. П.
# Дата:      22.10.2016.
# Версия:    0.1.

# TODO. 
# 1. Реализовать возможность пошагового исполнения процедур по нажатию
#    клавиши пользователем.
# 2. Выводить сообщения, отображающие значения чисел на каждом шаге
#    вычислений.

use strict;

my @operand1 = (); # массив цифр первого операнда
my @operand2 = (); # массив цифр второго операнда
my @result;        # массив результата работы функций
my $symbol;        # один символ, введенный с клавиатуры

# Функции, выполняющие конкретные операции, привяжем к знакам операций
# с помощью хеш-массива. Символы \& означают ссылку на функцию.
my %operations = ( '+' => \&addition, 
                   '-' => \&subtraction,
                   '*' => \&multiplication,
                   '/' => \&division 
                 );
my $operation;

print "Выберите операцию, введя один из символов: +, -, *, /\n";
$operation = <STDIN>;
chomp $operation;

# Используем экзотический способ проверки правильности введенного символа
# операции. Ищем введенный символ в списке допустимых символов. Функция
# grep() применяет блок кода {...} к каждому элементу массива и выводит 
# список тех элементов массива, для которых блок кода {...} возвратил true.
# Поэтому если длина полученного массива будет равна 0, значит, введенного
# символа нет в массиве допустимых символов. А длину массива можно проверить
# с помощью операции вида if ( @array == 0 ), поскольку в скалярном контексте 
# будет идти речь не о значениях элементов массива, а лишь о их числе.
if ( grep( { $operation eq $_; } ( '+', '-', '*', '/' ) ) == 0 )
{
  print "Введен неверный символ для выбора операции\n";
  exit( 1 );
}

# Сначала запросим у пользователя первый и второй операнды и представим их
# в виде массивов цифр.
# ВАЖНО. Цифры вводим в порядке от старших к младшим, т.е. так же, как пишем
#        числа на бумаге. В результате в этих массивах цифры будут
#        располагаться в таком порядке: старшие разряды будут иметь меньшие 
# индексы. Например, число 256: цифра 2 будет иметь индекс 0, цифра 5 -- 
# индекс 1, цифра 6 -- индекс 2.

# Запросим у пользователя цифры первого операнда (по одной за раз).

print "Введите цифры первого операнда (по одной за раз, нажимая Enter)\n";
print "Для завершения ввода числа нажмите Enter, не вводя цифру\n";

# Будем убирать символ новой строки '\n'.
while ( chomp( $symbol = <STDIN> ) )
{
  # Eсли просто нажали Enter -- конец ввода
  if ( $symbol =~ /^$/ )
  {
    last;
  }
  # Если ввели не ОДНУ ТОЛЬКО цифру -- повторить ввод.
  elsif ( $symbol !~ /^[0-9]$/ )
  {
    print "Вводить цифры нужно по одной\n";
    next;
  }

  push( @operand1, $symbol );      # Дополним массив новой цифрой.
}

# Запросим у пользователя цифры второго операнда (по одной за раз).

print "Введите цифры второго операнда (по одной за раз, нажимая Enter)\n";
print "Для завершения ввода числа нажмите Enter, не вводя цифру\n";

# Будем убирать символ новой строки '\n'.
while ( chomp( $symbol = <STDIN> ) )
{
  # Eсли просто нажали Enter -- конец ввода
  if ( $symbol =~ /^$/ )
  {
    last;
  }
  # Если ввели не ОДНУ ТОЛЬКО цифру -- повторить ввод.
  elsif ( $symbol !~ /^[0-9]$/ )
  {
    print "Вводить цифры нужно по одной\n";
    next;
  }

  push( @operand2, $symbol );      # Дополним массив новой цифрой.
}

# Выведем числа для контроля.
print "Первый операнд = " . join( "", @operand1 ) . "\n";
print "Второй операнд = " . join( "", @operand2 ) . "\n\n";

# Если есть лидирующие нули, то удалим их. Если же в числе изначально была 
# всего одна цифра -- нуль, то оставим его. Если в числе были одни нули,
# то оставим последний нуль.
while ( $operand1[ 0 ] == 0 && scalar @operand1 > 1 )
{
  shift( @operand1 );
}

while ( $operand2[ 0 ] == 0 && scalar @operand2 > 1 )
{
  shift( @operand2 );
}

# Выполняем выбранную операцию.
# Вызываем функцию по ее ссылке, хранящейся в хеш-массиве. В качестве
# параметров передаем ссылки на массивы цифр.
@result = &{ $operations{ $operation } }( \@operand1, \@operand2 );

if ( $operation eq '-' )
{
  # Печатаем результат. Возможен знак "минус".
  print "Разность = " . $result[ 1 ] . join( "", @{ $result[ 0 ] } ) . "\n";
}
elsif ( $operation eq '/' )
{
  # Печатаем результат.
  print "Частное = " . join( "", @{ $result[ 0 ] } ) . ", " .
        "остаток = " . join( "", @{ $result[ 1 ] } ) . "\n";
}

exit( 0 );

# -----------------------------------------------------------------------------
# Сложение
# -----------------------------------------------------------------------------
sub addition( $$ )
{
  print "Процерура сложения еще не реализована\n";
  return;
}

# -----------------------------------------------------------------------------
# Деление
# -----------------------------------------------------------------------------
sub division( $$ )
{
  my @dividend = @{ shift() };  # массив, содержащий цифры делимого
  my @divisor = @{ shift() };   # массив, содержащий цифры делителя

  my @result = ();              # массив, содержащий цифры результата
  my @remainder = ();           # массив, содержащий цифры остатка
  my $i;                        #
  my $digit;                    # отдельная цифра
  my $length_diff;              # разница в длине чисел
  my $which_is_bigger;          # результат сравнения чисел по величине
  my $pos_in_dividend;          # текущая позиция в массиве цифр делимого
  my @part_dividend;            # цифры неполного (частичного) делимого
  my $product_ref;              # ссылка на массив цифр произведения делителя
                                # на текущую цифру частного (ref -- reference)
  my @tmp;

  # Контрольная печать чисел.
  print "Делимое:  " . join( "", @dividend ) . "\n";
  print "Делитель: " . join( "", @divisor ) . "\n";

  # Деление на ноль запретим, но возвратим в качестве результата массив из
  # двух ссылок на анонимные массивы, состоящие из одного элемента каждый: 
  # первый  -- частное, второй -- остаток.
  if ( scalar @divisor == 1 && $divisor[ 0 ] == 0 )
  {
    print "Делить на ноль нельзя\n";
    return ( [ 0 ], [ 0 ] );
  }
  
  # Сравним числа по величине.
  $which_is_bigger = compare_numbers( \@dividend, \@divisor );

  if ( $which_is_bigger == 1 )     # делимое больше
  {
    ;   # Не делаем ничего (этот вариант показан лишь для наглядности).
  }
  elsif ( $which_is_bigger == 2 )  # делитель больше
  {
    # Результатом будет 0 (но передаем мы его в виде ссылки на анонимный
    # массив), а остатком будет все делимое (также передаем в виде ссылки).
    return ( [ 0 ], \@dividend );
  }
  else                             # числа одинаковые
  {
    # Возвращаем массив из двух ссылок на массивы.
    return ( [ 1 ], [ 0 ] );       # результат -- 1, остаток -- 0
  }
  
  # Берем цифры делимого одну за одной. В начале этой процедуры возможна
  # ситуация, когда частичное (неполное) делимое будет меньше делителя. 
  # Частичное делимое -- это несколько первых цифр делимого (а возможно, 
  # и только одна), взятых с его левого конца, т. е. старшие разряды. 
  # Так делают на бумаге при делении в столбик, так делаем и мы.
  for ( $pos_in_dividend = 0; $pos_in_dividend <= $#dividend; 
        $pos_in_dividend++ )
  {
    # Взятые цифры помещаем в массив @part_dividend.
    push( @part_dividend, $dividend[ $pos_in_dividend ] );

    # Возможна ситуация, когда при выполнении операции деления неполного
    # делимого на делитель оказывается, что произведение делителя на текущую
    # цифру частного в точности равно этому неполному делимому. При вычитании
    # этого произведения из неполного делимого получается 0. А следующие
    # цифры в делимом, которые нужно вводить в работу, тоже нули. В этом
    # случая нужно в частное добавить 0, а массив цифр неполного делимого
    # очистить, иначе у нас получится очередное неполное делимое, состоящее
    # из нулей, что не имеет смысла (и не будет корректно работать).
    # Пример: 10000 / 2.
    #
    # 10000 | 2
    # 10    |------ 
    # --     5000
    #  0 <-- этот нуль мы удаляем из массива @part_dividend и дальше
    #        не допускаем образования в массиве @part_dividend группы 
    #        нулей 0000 (т.е. нуль, полученный при вычитании и еще три нуля
    #        из делимого)
    if ( scalar @part_dividend == 1 && $part_dividend[ 0 ] == 0 )
    {
      push( @result, 0 );      # Добавим 0 в массив цифр частного.
      @part_dividend = ();     # Очистим (опустошим) массив.
      next;                    # Идем к началу цикла за новой цифрой делимого.
    }
    
    # Если делитель больше неполного делимого, то идем к началу цикла,
    # чтобы добавить в неполное делимое еще одну цифру.
    if ( compare_numbers( \@part_dividend, \@divisor ) == 2 )
    {
      # Добавим 0 в массив цифр частного. При этом если это еще только
      # первый неполный делитель (т.е. в частном еще нет ни одной ненулевой
      # цифры), то добавленный 0 окажется лидирующим (незначащим) нулем.
      # После завершения деления лидирующие нули нужно будет удалить
      # из частного.
      push( @result, 0 );
      next;
    }
  
    # Подберем очередную цифру частного и вычислим произведение этой цифры
    # и делителя.
    ( $digit, $product_ref ) = get_quotient_digit( \@part_dividend,
                                                   \@divisor );
    # Поместим цифру в результирующий массив.
    push( @result, $digit );

    # Вычтем из частичного делимого полученное произведение и запишем разность
    # опять в массив частичного делимого.
    @tmp = subtraction( \@part_dividend, $product_ref );
    @part_dividend = @{ $tmp[ 0 ] };

    # Если получилось совпадение частичного делимого и произведения текущей
    # цифры частного и делителя, то в результате их вычитания получим 0.
    # Пример: 121 разделить на 3. После первого шага получим 12 - 12 = 0,
    # а в частное идет цифра 4.
    # В таком случае этот единственный 0 нужно вообще убрать из массива @tmp.
    # При делении в столбик на бумаге такой 0 тоже не сохраняют, поскольку
    # он не является значащей цифрой.
    # Однако если мы включили в частичное делимое уже последнюю цифру исходного
    # делимого, тогда удалять этот 0 из массива @part_dividend не нужно. 
    # При делении в столбик также оставляют 0, если он получился в результате 
    # последнего вычитания. 
    # Пример:
    # 124 | 4
    # 12  |-----
    # --- | 31 
    #   4
    #   4
    #   --
    #   0
    if ( scalar @part_dividend == 1 && $part_dividend[ 0 ] == 0 &&
         $pos_in_dividend != $#dividend )
    {
      # Если текущая цифра делимого -- не последняя ($pos_in_dividend != 
      # $#dividend), то очистим массив.
      @part_dividend = ();
    }
  }

  # Все цифры делимого обработаны, нужно подводить итоги.

  # Если этот массив пустой, значит, мы очищали его, когда очередная
  # (последняя) цифра делимого оказалась нулем. Деление получилось 
  # без остатка, поэтому ...
  if ( scalar @part_dividend == 0 )
  {
    @remainder = ( 0 );  # ... поместим в массив цифр остатка цифру 0
  }
  # Если деление получилось без остатка, ...
  elsif ( scalar @part_dividend == 1 && $part_dividend[ 0 ] == 0 )
  {
    @remainder = ( 0 );  # ... поместим в массив цифр остатка цифру 0
  }
  else
  {
    @remainder = @part_dividend;    # Запишем неполное делимое в остаток.
  }

  # После завершения деления лидирующие нули удалим из частного. Откуда эти
  # нули берутся -- см. комментарий выше.
  shift( @result ) 
    while ( $result[ 0 ] == 0 );

  # Возвратим массив из двух ссылок на массивы: частное и остаток.
  return ( \@result, \@remainder );
}

# -----------------------------------------------------------------------------
# Умножение
# -----------------------------------------------------------------------------
sub multiplication( $$ )
{
  print "Процерура умножения еще не реализована\n";
  return;
}

# -----------------------------------------------------------------------------
# Вычитание
# -----------------------------------------------------------------------------
sub subtraction( $$ )
{
  # Получим значения фактических аргументов функции с помощью функции shift.
  # Функция shift при вызове ее без параметров работает с массивом @_. Если
  # эта работа происходит внутри пользовательской функции, то в массиве @_ 
  # содержатся параметры пользовательской функции. Поскольку параметрами 
  # являются ссылки на массивы, то необходимо из них сделать массивы 
  # с помощью конструкции @{ ... }.
  my @minuend = @{ shift() };    # массив, содержащий цифры уменьшаемого
  my @subtrahend = @{ shift() }; # массив, содержащий цифры вычитаемого

  my @result = ();             # массив, содержащий цифры результата
  my $i;                       #
  my $digit;                   # отдельная цифра
  my $length_diff;             # разница в длине чисел
  my $which_is_bigger;         # результат сравнения чисел по величине
  my @tmp;
  my $sign = "";               # знак полученного результата
                               # (если уменьшаемое меньше вычитаемого,
                               # то будет "-", в противном случае останется "")

  # Сравним числа по величине.
  $which_is_bigger = compare_numbers( \@minuend, \@subtrahend );

  if ( $which_is_bigger == 1 )     # уменьшаемое больше
  {
    ;   # Не делаем ничего (этот вариант показан лишь для наглядности).
  }
  elsif ( $which_is_bigger == 2 )  # вычитаемое больше
  {
    # Поменяем местами уменьшаемое и вычитаемое.
    @tmp = @minuend;
    @minuend = @subtrahend;
    @subtrahend = @tmp;
    $sign = "-";                   # знак результата будет "минус"
  }
  else                             # числа одинаковые
  {
    # [ 0 ] --  ссылка на анонимный массив, содержащий один элемент
    return ( [ 0 ], $sign );
  }

  # Вычислим разницу в длине чисел.
  $length_diff = scalar @minuend - scalar @subtrahend;

  # Если вычитаемое короче уменьшаемого, дополним вычитаемое нулями спереди.
  for ( my $j = 0; $j < $length_diff; $j++ )
  {
    # Вставляем нули в массив слева (с начала массива).
    unshift( @subtrahend, 0 );
  }

  # Контрольная печать чисел.
  print join( "", @minuend ) . "\n";
  print join( "", @subtrahend ) . "\n";

  # Индекс последнего элемента в массиве цифр вычитаемого. Это будет индекс
  # самой младшей цифры.
  $i = $#minuend;

  # пока не вышли за левый край массива
  while ( $i >= 0 )
  {
    # Если текущая цифра уменьшаемого НЕ МЕНЬШЕ текущей цифры вычитаемого,
    # тогда все просто -- берем разность этих цифр и результирующую цифру
    # записываем в массив результата.
    if ( $minuend[ $i ] >= $subtrahend[ $i ] )
    {
      # ПРИМЕЧАНИЕ. Массив @result был пустым при его создании. Однако
      #             при записи в него значения в позицию, превышающую его 
      #             длину, массив автоматически удлиняется.
      $result[ $i ] = $minuend[ $i ] - $subtrahend[ $i ];

      # Для перехода к следующему разряду уменьшаем индекс, т. к. мы идем 
      # от младших разрядов к старшим, но младшие разряды располагаются 
      # в элементах массива с бОльшими индексами.
      $i--;
    }
    # Если текущая цифра уменьшаемого МЕНЬШЕ текущей цифры вычитаемого,
    # но предыдущая цифра уменьшаемого больше нуля, тогда мы можем "занять"
    # единицу в этом более старшем разряде.
    elsif ( $minuend[ $i - 1 ] > 0 )
    {
      # Для проведения вычислений прибавим к цифре уменьшаемого 10, ...
      $result[ $i ] = $minuend[ $i ] + 10 - $subtrahend[ $i ];

      # ... а предыдущую цифру уменьшаемого уменьшим на 1.
      $minuend[ $i - 1 ]--;

      $i--;     # переходим к следующему разряду
    }
    # Если же предыдущая цифра уменьшаемого равна нулю, ...
    else
    {
      my $tmp_index = $i - 1;  # предыдущая позиция в уменьшаемом

      # ... тогда будем искать более старшую позицию, в которой стоит цифра, 
      # отличная от нуля, ...
      while ( $minuend[ $tmp_index ] == 0 )
      {
        # ... а во всех этих позициях вместо нулей будем ставить цифру 9.
        $minuend[ $tmp_index ] = 9;
        $tmp_index--;
      }

      # Нашли позицию, в которой стоит не 0. Уменьшим эту цифру на 1 (мы ведь
      # "заняли" здесь единицу).
      $minuend[ $tmp_index ]--;

      # Получаем очередную цифру результата так же, как и в предыдущем случае.
      $result[ $i ] = $minuend[ $i ] + 10 - $subtrahend[ $i ];

      # Эта операция не нужна, т. к. мы уже записали в эту позицию цифру 9,
      # тем самым вычли 1 из цифры, стоящей в уменьшаемом в этой позиции.
      # $minuend[ $i - 1 ]--;

      $i--;    # переходим к следующему разряду
    }
  }

  # В цикле удаляем лидирующие нули из числа-результата, но последний нуль
  # оставляем.
  shift( @result ) 
    while ( $result[ 0 ] == 0 && scalar @result > 1 );

  # Возвращаем ссылку на массив цифр результата и знак результата.
  return ( \@result, $sign );
}

# -----------------------------------------------------------------------------
# Вспомогательные процедуры
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
# Сравнение двух чисел
# -----------------------------------------------------------------------------
sub compare_numbers( $$ )
{
  my @num1 = @{ shift() };         # массив, содержащий цифры первого числа
  my @num2 = @{ shift() };         # массив, содержащий цифры второго числа

  my $digits_num;                  # число цифр в числах (если длины одинаковые)
  my $result;
  my $pos;                         # текущая позиция в числе (индекс в массиве)

  # ВАЖНО!
  # Считаем, что лидирующих (незначащих) нулей в числах НЕТ.
  # ---------------------------------------------------------
  
  # Первое число длиннее второго -- оно больше.  
  if ( scalar @num1 > scalar @num2 )
  {
    $result = 1;
  }
  # Второе число длиннее первого -- оно больше.
  elsif ( scalar @num1 < scalar @num2 )
  {
    $result = 2;
  }
  else  # длины чисел равны
  {
    $pos = 0;
    $digits_num = scalar @num1;  # количество цифр в каждом из чисел

    # Сравниваем цифры в одной и той же позиции, пока цифры не закончились.
    # Продвигаемся от старших разрядов к младшим.
    while ( $pos < $digits_num && $num1[ $pos ] == $num2[ $pos ] )
    {
      # Если цифры обоих чисел, стоящие в одной и той же позиции, одинаковые,
      # то ничего не делаем, ...
      $pos++;   # ... а только наращиваем индекс позиции
    }

    # Если проверили все цифры, и различий между цифрами, стоящими в одной
    # позиции, не было, значит, числа равны.
    if ( $pos == $digits_num )
    {
      $result = 0;
    }
    else  # $pos < $digits_num, т. е. различие было найдено в позиции $pos
    {
      if ( $num1[ $pos ] > $num2[ $pos ] )
      {
        $result = 1;  # Первое число больше второго.
      }
      else
      {
        $result = 2;  # Второе число больше первого.
      }
    }
  }

  return $result;
}

# -----------------------------------------------------------------------------
# Получение очередной цифры частного
# -----------------------------------------------------------------------------
sub get_quotient_digit( $$ )
{
  my @part_dividend = @{ shift() }; # массив, содержащий цифры частичного
                                    # делимого
  my @divisor = @{ shift() };       # массив, содержащий цифры делителя

  my $multiplier;                   # пробная цифра частного (множитель)
  my @product = ();                 # произведение этой цифры на делитель
  my $two_digits_product;           # произведение этой цифры на текущую
                                    # цифру делителя
  my $additional_digit = 0;         # цифра старшего разряда, которая может
                                    # возникать, если в $two_digits_product
                                    # получилось двузначное число
  my $pos;                          # текущая позиция в делителе
  
  # Цифру частного получаем методом подбора. Начинаем с цифры 9. Умножаем
  # делитель на нее. Если полученное произведение БОЛЬШЕ неполного делимого,
  # берем цифру 8 и т. д. Как только полученное произведение будет НЕ БОЛЬШЕ
  # неполного делимого -- цифра найдена!
  for ( $multiplier = 9; $multiplier >= 1; $multiplier-- )
  {
    # Берем цифры делителя одну за другой, начиная с самого младшего разряда.
    for ( $pos = $#divisor; $pos >= 0; $pos-- )
    {
      # К произведению цифры-кандидата и текущей цифры делителя добавляем
      # цифру, которую мы "держали в уме", как это делается при умножении
      # в столбик. 
      $two_digits_product = $multiplier * $divisor[ $pos ] + $additional_digit;

      # Поместим в результирующий массив цифр произведения остаток от деления
      # на 10. Если даже произведение оказалось меньше 10, это сработает.
      push( @product, $two_digits_product % 10 );

      # Если же произведение двух цифр оказалось больше 10, ...
      if ( $two_digits_product >= 10 )
      {
        # ... возьмем его первую цифру, и она будет у нас "в уме".
        $additional_digit = substr( sprintf( "%2d", $two_digits_product ),
                                    0, 1 );
      }
      # В противном случае ...
      else
      {
        # ... обнулим эту цифру "в уме", чтобы не осталось ее значения
        # от предыдущей итерации.
        $additional_digit = 0;
      }
    }

    # Если эта цифра "в уме" оказалась больше нуля после обработки последней 
    # цифры делителя, ...
    if ( $additional_digit > 0 )
    {
      # ... поместим ее в результирующий массив цифр произведения. 
      # Делаем все -- как на бумаге при умножении в столбик.
      push( @product, $additional_digit );
    }

    # Поскольку мы помещали цифры произведения в массив, начиная с младших
    # разрядов, то нужно переписать массив в обратном порядке, чтобы в позициях
    # с младшими индексами находились старшие разряды числа.
    @product = reverse( @product );
    
    # Как только произведение станет меньше частичного делимого или равно
    # ему, сразу прекращаем цикл.
    last if ( compare_numbers( \@part_dividend, \@product ) == 1 || 
              compare_numbers( \@part_dividend, \@product ) == 0 );

    # Перед взятием новой цифры-кандидата очистим переменные.
    @product = ();
    $additional_digit = 0;
  }

  # Возвращаем массив с двумя элементами: очередной цифрой частного
  # и ссылкой на массив цифр произведения делителя на эту цифру.
  return ( $multiplier, \@product );
}
