#!/usr/bin/perl -w

# -----------------------------------------------------------------------------
# Программа: "Регулярные выражения"
# Автор:     Моргунов Е. П.
# Дата:      03.11.2013
# Версия:    0.1
# -----------------------------------------------------------------------------

# эта строка означает строгую проверку синтаксиса исходного текста программы
use strict;

# в этом хеш-массиве ключами будут номера пунктов меню, а значениями -- ссылки
# на соответствующие процедуры
my %procedures = ( 1 => \&simple_regular_expressions,
                   2 => \&groupings,
                   3 => \&simple_substitutions );

while ( 1 )
{
  # Это так называемый here-document -- "документ здесь" (см. руководство perlop,
  # команда -- man perlop). Пробелы между символами "<<" и идентификатором 
  # недопустимы. Завершающий идентификатор пишется, начиная с самой первой 
  # позиции в строке. В качестве идентификатора может быть взята и другая строка,
  # например, EOF или FINISH, или END и т. п.
  print <<END_OF_MENU;
  1 -- Простые регулярные выражения: поиск подстрок
  2 -- Группировки символов
  3 -- Простые замены символов
  q -- выход из программы
END_OF_MENU

  my $option = <STDIN>;
  chomp $option;
  
  if ( $option eq 'q' )   # завершение работы
  {
    last;                 # выход из вечного цикла
  }
  # существует ли в хеш-массиве такой ключ (пункт меню)
  elsif ( exists $procedures{ $option } )
  {
    # если ключ существует, то вызываем процедуру, соответствующую ему (в хеш-
    # массиве хранится ссылка на эту процедуру) 
    &{ $procedures{ $option } }();
  }
  else                    # неверный пункт меню 
  {
    next;                 # вернемся к началу цикла -- выводу меню
  }
}

exit( 0 );


# -----------------------------------------------------------------------------
# Простые регулярные выражения: поиск подстрок
# -----------------------------------------------------------------------------
sub simple_regular_expressions
{
  my @names = qw( Andrew Eugene Paul Jonathan Caroline John Antony Natalia
                  Olivia Ophelia Roland Ronald );

  # это то же самое, что и 
  # my @names = ( 'Andrew', 'Eugene', 'Paul', 'Jonathan', 'Caroline', 'John',
  #               'Antony', 'Natalia', 'Olivia', 'Ophelia', 'Roland', 
  #               'Ronald' );

  print "Массив имен: ";

  # преобразуем массив в строку, попутно отсортировав его, и напечатаем
  print join( " ", sort @names );
  print "\n\n";

  print "Выбрать имена, начинающиеся с символов 'An'\n";
  print "Регулярное выражение -- /^An/\n";
  
  foreach my $name ( sort @names )
  {
    print "$name " if ( $name =~ /^An/ );
  }
  print "\n\n";

  # -----------------------------------

  print "Выбрать имена, состоящие из четырех букв\n";
  # Символы "^" и "$" -- не являются хранимыми. Они означают так называемую
  # привязку регулярного выражения к началу строки ("^") и к концу строки ("$").
  # В данном случае это означает, что в том имени, которое нас устроит, должно
  # быть ровно 4 буквы, причем, первая -- заглавная буква. Комбинация [A-Z]
  # означает -- любая заглавная буква от A до Z, [a-z] -- маленькая буква, 
  # {3} -- число маленьких букв.
  # Обратите внимание, что при выводе рег. выражения на экран мы ставим символ
  # "\" перед символом "$", чтобы он был выведен буквально.
  print "Регулярное выражение -- /^[A-Z][a-z]{3}\$/\n";
  
  foreach my $name ( sort @names )
  {
    print "$name " if ( $name =~ /^[A-Z][a-z]{3}$/ );
  }
  print "\n\n";

  # -----------------------------------

  print "Выбрать имена, завершающиеся символами 'ia'\n";
  print "Регулярное выражение -- /ia\$/\n";
  
  foreach my $name ( sort @names )
  {
    print "$name " if ( $name =~ /ia$/ );
  }
  print "\n\n";

  # -----------------------------------

  print "Выбрать имена, начинающие с 'J' и завершающиеся символом 'n'\n";
  print "Регулярное выражение -- /^J[a-z]+n\$/ или /^J.+n\$/\n";
  
  foreach my $name ( sort @names )
  {
    # Знак "+" -- это квантификатор, означает -- один и более символов, стоящих
    # перед ним.
    print "$name " if ( $name =~ /^J[a-z]+n$/ );

    # символ "." означает любой символ, в т. ч. и букву латинского алфавита
    print "$name " if ( $name =~ /^J.+n$/ );
  }
  print "\n\n";

  # -----------------------------------
  
  print "Выбрать имена Roland и Ronald\n";
  print "Регулярное выражение -- /^Ro(lan|nal)d\$/\n";
  
  foreach my $name ( sort @names )
  {
    # Символы (lan|nal) означают -- альтернативные варианты: либо "lan", либо 
    # "nal".
    print "$name " if ( $name =~ /^Ro(lan|nal)d$/ );
  }
  print "\n\n";

  return;
}

# -----------------------------------------------------------------------------
# группировки элементов строки 
# -----------------------------------------------------------------------------
sub groupings
{
  # Даты с использованием в качестве разделителей дня, месяца и года различных
  # символов: дефис, косая черта и точка.
  my $date1 = "03-11-2013"; 
  my $date2 = "03/11/2013"; 
  my $date3 = "03.11.2013"; 

  # Задача: вычленить из даты ее элементы.
  
  # Символы ^ и $ -- привязки к началу и к концу строки (они не являются частью
  # значения переменной). Выражения в скобках, например, "(\d{1,2})" -- это
  # группировки символов. После вычисления регулярного выражения эти группы
  # символов записываются во встроенные переменные языка Perl с именами $1, $2,
  # $3 и т. д. В нашем случае таких переменных -- три. И еще, символ \d означает
  # любую цифру, а {1,2} -- квантификатор, означающий, что цифр может быть от 
  # одной до двух.
  $date1 =~ /^(\d{1,2})-(\d{1,2})-(\d{4})$/;
  
  print "Дата \$date1: день -- \$1, месяц -- \$2, год -- \$3\n";
  print "Дата $date1: день -- $1, месяц -- $2, год -- $3\n";
  print "\n\n";

  # Для того, чтобы учесть возможность использования различных символов,
  # разделяющих элементы дат, используем такое выражение (в нем [-\/\.] --
  # это класс символов, при этом нужно перед косой чертой и точкой поставить
  # символ "\", чтобы и точка, и косая черта воспринимались буквально, а не
  # как специальные символы):
  $date2 =~ /^(\d{1,2})[-\/\.](\d{1,2})[-\/\.](\d{4})$/;
  
  print "Дата $date2: день -- $1, месяц -- $2, год -- $3\n";
  print "\n\n";

  # ВАЖНО! Переменные $1, $2 и т.д. сохраняют свои значения ТОЛЬКО до вычисления
  #        следующего регулярного выражения.

  # Еще один способ разделения даты на элементы.
  my @date_elements = $date3 =~ /^(\d{1,2})[-\/\.](\d{1,2})[-\/\.](\d{4})$/;

  print "Дата \$date3: день -- \$date_elements[ 0 ], " .
        "месяц -- \$date_elements[ 1 ], год -- \$date_elements[ 2 ]\n";
  print "Дата $date3: день -- $date_elements[ 0 ], " .
        "месяц -- $date_elements[ 1 ], год -- $date_elements[ 2 ]\n";
  print "\n\n";

  # И еще один способ разделения даты на элементы. Каждая группа символов
  # записывается в соответствующую переменную: первая группа -- в первую
  # переменную, вторая -- во вторую и т.д. 
  my( $day, $month, $year );
  
  ( $day, $month, $year ) = 
     $date3 =~ /^(\d{1,2})[-\/\.](\d{1,2})[-\/\.](\d{4})$/;

  print "Дата \$date3: день -- \$day, месяц -- \$month, год -- \$year\n";
  print "Дата $date3: день -- $day, месяц -- $month, год -- $year\n";
  print "\n\n";

  return;
}

# -----------------------------------------------------------------------------
# Простые замены символов
# -----------------------------------------------------------------------------
sub simple_substitutions
{
  # Задача: преобразовать дату из формата "день-месяц-год" в формат
  #         "год-месяц-день".
  
  my $date = "03-11-2013"; 
  my $new_date;
  
  $new_date = $date;

  # В этом рег. выражении формат такой -- s///. Символ "s" означает --
  # substitute -- заменить, подставить. В выражении уже три символа "/",
  # поэтому образуется две части выражения. В первой происходит поиск
  # нужной комбинации символов, а во второй -- какое-то преобразование,
  # например, перестановка групп символов местами или отбрасывание символов,
  # т.е. замена их пустой строкой.
  $new_date =~ s/^(\d{1,2})[-\/\.](\d{1,2})[-\/\.](\d{4})$/$3-$2-$1/;
  
  print "Дата $date, новый формат - $new_date\n";
  print "\n\n";

  # Получим из полного значения даты только год, отбросив день и месяц.
  my $year = $date;
  $year =~ s/^(\d{1,2})[-\/\.](\d{1,2})[-\/\.]//;

  print "Дата $date, год - $year\n";
  print "\n\n";

  return;
}
