/* Пример 6.14 из книги:

   Рэндал Э. Брайант, Дэвид Р. О'Халларон. Компьютерные системы. Архитектура
   и программирование. СПб. : БХВ-Петербург, 2005.
*/

/* Умножение матриц. Пространственная локальность. */

/*
  Компилировать с параметром -pg

  Запуск:
  ./mul_matrices size
  Например, 
  ./mul_matrices 500

  Для профилировки (с выводом результатов в файл a.prof)
  
  gprof mul_matrices > a.prof
*/

#include <stdio.h>
#include <stdlib.h>


void mul_ijk( int **a, int **b, int **c, int size );
void mul_jik( int **a, int **b, int **c, int size );
void mul_jki( int **a, int **b, int **c, int size );
void mul_kji( int **a, int **b, int **c, int size );
void mul_kij( int **a, int **b, int **c, int size );
void mul_ikj( int **a, int **b, int **c, int size );

long long sum_matrix_rows( int **c, int size );
void init_c( int **c, int size );


int main( int argc, char **argv )
{
  int i;
  int j;

  int **matrix_a;
  int **matrix_b;
  int **matrix_c;
  
  /* матрицы квадратные */
  int size = atoi( argv[ 1 ] );  /* число строк и столбцов в матрице */

  /* суммы элементов результирующей матрицы-произведения */
  long long ijk; 
  long long jik;
  long long jki;
  long long kji; 
  long long kij;
  long long ikj;
  
  /* Сначала выделим память под указатели на будущие строки матрицы. */
  matrix_a = ( int **)malloc( size * sizeof( int * ) );
  
  /* Для каждой строки ... */
  for ( i = 0; i < size; i++ )
  {
    /* ... выделим память. */
    matrix_a[ i ] = ( int *)malloc( size * sizeof( int ) );
    
    /* Значения всех элементов строки будут равны ее номеру. */
    for ( j = 0; j < size; j++ )
    {
      matrix_a[ i ][ j ] = i;
    }
  }


  /* Сначала выделим память под указатели на будущие строки матрицы. */
  matrix_b = ( int **)malloc( size * sizeof( int * ) );
  
  /* Для каждой строки ... */
  for ( i = 0; i < size; i++ )
  {
    /* ... выделим память. */
    matrix_b[ i ] = ( int *)malloc( size * sizeof( int ) );
    
    /* Значения всех элементов строки будут равны ее номеру. */
    for ( j = 0; j < size; j++ )
    {
      matrix_b[ i ][ j ] = i;
    }
  }

  /* Сначала выделим память под указатели на будущие строки матрицы. */
  matrix_c = ( int **)malloc( size * sizeof( int * ) );
  
  /* Для каждой строки ... */
  for ( i = 0; i < size; i++ )
  {
    /* ... выделим память. */
    matrix_c[ i ] = ( int *)malloc( size * sizeof( int ) );
    
    /* Значения всех элементов строки будут равны 0. */
    for ( j = 0; j < size; j++ )
    {
      matrix_c[ i ][ j ] = 0;
    }
  }

  /* Умножаем матрицы. */
  mul_ijk( matrix_a, matrix_b, matrix_c, size );
  
  /* Находим сумму всех элементов матрицы для контроля. */
  ijk = sum_matrix_rows( matrix_c, size );   /* суммирование по строкам */
  printf( "ijk = %lld\n", ijk );
  init_c( matrix_c, size );                  /* Обнуляем элементы матрицы. */

  /* Умножаем матрицы. */
  mul_jik( matrix_a, matrix_b, matrix_c, size );

  /* Находим сумму всех элементов матрицы для контроля. */
  jik = sum_matrix_rows( matrix_c, size );
  printf( "jik = %lld\n", jik );
  init_c( matrix_c, size );

  /* Далее -- аналогично. */
  mul_jki( matrix_a, matrix_b, matrix_c, size );
  jki = sum_matrix_rows( matrix_c, size );
  printf( "jki = %lld\n", jki );
  init_c( matrix_c, size );

  mul_kji( matrix_a, matrix_b, matrix_c, size );
  kji = sum_matrix_rows( matrix_c, size );
  printf( "kji = %lld\n", kji );
  init_c( matrix_c, size );

  mul_kij( matrix_a, matrix_b, matrix_c, size );
  kij = sum_matrix_rows( matrix_c, size );
  printf( "kij = %lld\n", kij );
  init_c( matrix_c, size );

  mul_ikj( matrix_a, matrix_b, matrix_c, size );
  ikj = sum_matrix_rows( matrix_c, size );
  printf( "ikj = %lld\n", ikj );
  /* init_c( c, size ); */

  return 0;
}


void mul_ijk( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int sum;

  for ( i = 0; i < size; i++ )
  {
    for ( j = 0; j < size; j++ )
    {
      sum = 0;
      for ( k = 0; k < size; k++ )
      {
        sum += a[ i ][ k ] * b[ k ][ j ];
      }
      c[ i ][ j ] += sum;
    }
  }

  return;
}

void mul_jik( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int sum;

  for ( j = 0; j < size; j++ )
  {
    for ( i = 0; i < size; i++ )
    {
      sum = 0;
      for ( k = 0; k < size; k++ )
      {
        sum += a[ i ][ k ] * b[ k ][ j ];
      }
      c[ i ][ j ] += sum;
    }
  }

  return;
}

void mul_jki( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int r;
  
  for ( j = 0; j < size; j++ )
  {
    for ( k = 0; k < size; k++ )
    {
      r = b[ k ][ j ];
      for ( i = 0; i < size; i++ )
      {
        c[ i ][ j ] += a[ i ][ k ] * r;
      }
    }
  }

  return;
}

void mul_kji( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int r;
  
  for ( k = 0; k < size; k++ )
  {
    for ( j = 0; j < size; j++ )
    {
      r = b[ k ][ j ];
      for ( i = 0; i < size; i++ )
      {
        c[ i ][ j ] += a[ i ][ k ] * r;
      }
    }
  }

  return;
}

void mul_kij( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int r;
  
  for ( k = 0; k < size; k++ )
  {
    for ( i = 0; i < size; i++ )
    {
      r = a[ i ][ k ];
      for ( j = 0; j < size; j++ )
      {
        c[ i ][ j ] += r * b[ k ][ j ];
      }
    }
  }

  return;
}

void mul_ikj( int **a, int **b, int **c, int size )
{
  int i;
  int j;
  int k;
  int r;
  
  for ( i = 0; i < size; i++ )
  {
    for ( k = 0; k < size; k++ )
    {
      r = a[ i ][ k ];
      for ( j = 0; j < size; j++ )
      {
        c[ i ][ j ] += r * b[ k ][ j ];
      }
    }
  }

  return;
}

void init_c( int **matrix_c, int size )
{
  int i;
  int j;
  
  /* Для каждой строки ... */
  for ( i = 0; i < size; i++ )
  {
    /* ... значения всех элементов строки будут равны 0. */
    for ( j = 0; j < size; j++ )
    {
      matrix_c[ i ][ j ] = 0;
    }
  }
  return;
}

long long sum_matrix_rows( int **matrix_c, int size )
{
  int i;
  int j;
  long long sum = 0;

  for ( i = 0; i < size; i++ )
  {
    for ( j = 0; j < size; j++ )
    {
      /* При суммировании идем вдоль строки. */
      sum += matrix_c[ i ][ j ];
    }
  }

  return sum;
}
