/* Пример 6.2 из книги:

   Рэндал Э. Брайант, Дэвид Р. О'Халларон. Компьютерные системы. Архитектура
   и программирование. СПб. : БХВ-Петербург, 2005.
*/

/* Суммирование элементов матрицы. Пространственная локальность. */

/*
  Компилировать с параметром -pg

  Запуск:
  ./sum_arrays num_rows num_cols
  Например, 
  ./sum_arrays 5000 10000

  Для профилировки (с выводом результатов в файл a.prof)
  
  gprof sum_arrays > a.prof
*/

#include <stdio.h>
#include <stdlib.h>


int sum_array_rows( int **a, int rows, int cols );
int sum_array_cols( int **a, int rows, int cols );


int main( int argc, char **argv )
{
  int i;
  int j;

  int **array;
  
  int rows = atoi( argv[ 1 ] );  /* число строк в матрице */
  int cols = atoi( argv[ 2 ] );  /* число столбцов в матрице  */

  /* сумма всех элементов матрицы */
  long sum_rows = 0;  /* суммирование по строкам */
  long sum_cols = 0;  /* суммирование по столбцам */
  
  /* Cначала выделим память под указатели на будущие строки матрицы. */
  array = ( int ** )malloc( rows * sizeof( int * ) );
  
  /* Для каждой строки ... */
  for ( i = 0; i < rows; i++ )
  {
    /* ... выделим память. */
    array[ i ] = ( int * )malloc( cols * sizeof( int ) );
    
    /* Значения всех элементов строки будут равны ее номеру. */
    for ( j = 0; j < cols; j++ )
    {
      array[ i ][ j ] = i;
    }
  }

  /* Находим сумму всех элементов матрицы. */

  /* суммирование по строкам */
  sum_rows = sum_array_rows( array, rows, cols );

  /* суммирование по столбцам */
  sum_cols = sum_array_cols( array, rows, cols );

  printf( "Сумма по строкам = %ld\n", sum_rows );
  printf( "Сумма по столбцам = %ld\n", sum_cols );

  return 0;
}


int sum_array_rows( int **a, int rows, int cols )
{
  int i;
  int j;
  int sum = 0;

  for ( i = 0; i < rows; i++ )
  {
    for ( j = 0; j < cols; j++ )
    {
      /* При суммировании идем вдоль строки. */
      sum += a[ i ][ j ];
    }
  }

  return sum;
}

int sum_array_cols( int **a, int rows, int cols )
{
  int i;
  int j;
  int sum = 0;
  
  for ( j = 0; j < cols; j++ )
  {
    for ( i = 0; i < rows; i++ )
    {
      /* При суммировании переходим от одной строки к другой, выбирая
         элемент из того же столбца. */
      sum += a[ i ][ j ];
    }
  }

  return sum;
}
