# -----------------------------------------------------------------------------
# Модуль Perl: функции для проведения геометрических вычислений
# -----------------------------------------------------------------------------

package Math::Geometry;  # каталог Math, файл Geometry.pm

use strict;
use warnings;

BEGIN
{
  use Exporter   ();
  our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);

  # установим номер версии
  $VERSION     = 1.00;
  # эта строка может быть полезна при использовании
  # систем управления версиями (RCS/CVS)
  $VERSION = sprintf "%d.%03d", q$Revision: 1.1 $ =~ /(\d+)/g;

  @ISA         = qw( Exporter );

  # экспортируемые функции модуля (пакета),
  # которые экспортируются по умолчанию
  # ПРИМЕЧАНИЕ. Обратите внимание, что мы включили в этот массив только
  #             две наших функции: circle_area и circle_len. Это сделано
  #             в учебно-методических целях.
  @EXPORT      = qw( &func1 &func2 &func4 
                     &circle_area &circle_len );
  %EXPORT_TAGS = ();     # например: TAG => [ qw!name1 name2! ],

  # экспортируемые глобальные переменные модуля (пакета)
  # и функции, которые НЕ экспортируются по умолчанию
  # ПРИМЕЧАНИЕ. Обратите внимание, что мы включили в этот массив только
  #             одну нашу функцию square_perim. Это сделано
  #             в учебно-методических целях.
  @EXPORT_OK   = qw( $Var1 %Hashit &func3
                     &square_perim );
}
our @EXPORT_OK;

# экспортируемые глобальные переменные данного модуля (пакета)
our $Var1;
our %Hashit;

# неэкспортируемые глобальные переменные данного модуля (пакета)
our @more;
our $stuff;

# сначала инициализируем экспортируемые глобальные переменные модуля (пакета)
$Var1   = '3';
%Hashit = ();

# затем инициализируем остальные глобальные переменные модуля (пакета)
# ПРИМЕЧАНИЕ. Эти переменные доступны извне пакета в форме, например,
#             $Math::Geometry::stuff, а не просто $stuff.
$stuff  = '4';
@more   = ();

# все лексические переменные, область видимости которых ограничена
# данным файлом, должны быть созданы прежде, чем созданы функции,
# использующие эти переменные

# это лексические переменные, видимые в пределах этого файла
my $priv_var    = '2';
my %secret_hash = ();

# здесь размещаются функции, видимые в пределах этого файла
# вызываются такие функции так: &$priv_func;
# такие функции не могут иметь прототипа
my $priv_func = sub {
    # текст функции
};

# здесь располагаются все остальные функции, как экспортируемые,
# так и неэкспортируемые
# ПРИМЕЧАНИЕ. Не забудьте включить какой-нибудь осмысленный текст
#             внутри скобок {}.
sub func1      {}    # функция без прототипа
sub func2()    {}    # прототип указывает на отсутствие параметров
sub func3($$)  {}    # прототип указывает на наличие двух скалярных параметров
sub func4(\%)  {}    # прототип указывает на наличие параметра --
                     # ссылки на хеш-массив

END { }       # код, выполняющийся при завершении работы модуля
              # (глобальный деструктор)


# Здесь располагается ваш программный код
# ========================================

# ----------------------------------------------------------------------------
# функции для работы с кругом и окружностью
# ----------------------------------------------------------------------------

use constant   PI => 3.14159265;

# ---------------------------
# вычисление площади круга
# ---------------------------
sub circle_area( $$ )
{
  # параметры - радиус окружности и признак вывода формулы на экран
  my $rad = shift;
  my $formula = shift;

  my $area;

  # площадь равна: пи * радиус в квадрате
  $area = PI * $rad * $rad;
  
  if ( $formula )
  {
    print "Формула для расчета площади круга: \n" .
          "--- пи * радиус в квадрате ---\n";
  }
  
  return ( $area );  # краткая запись: return ( PI * $rad * $rad );
}

# -------------------------------
# вычисление длины окружности
# -------------------------------
sub circle_len( $$ )
{
  # параметры - радиус окружности и признак вывода формулы на экран
  my $rad = shift;
  my $formula = shift;

  my $len;

  # длина окружности равна: 2 * пи * радиус
  $len = 2 * PI * $rad;

  if ( $formula )
  {
    print "Формула для расчета длины окружности: \n" .
          "--- 2 * пи * радиус ---\n";
  }
  
  return ( $len );
}


# ----------------------------------------------------------------------------
# функции для работы с квадратом
# ----------------------------------------------------------------------------

# -----------------------------
# вычисление площади квадрата
# -----------------------------
sub square_area( $$ )
{
  # параметры - сторона квадрата и признак вывода формулы на экран
  my $side = shift;
  my $formula = shift;

  my $area;
  
  # площадь равна: длина стороны в квадрате
  $area = $side * $side;

  if ( $formula )
  {
    print "Формула для расчета площади квадрата: \n" .
          "--- длина стороны в квадрате ---\n";
  }

  return ( $area );
}

#----------------------------------
# вычисление периметра квадрата
# ---------------------------------
sub square_perim( $$ )
{
  # параметры - сторона квадрата и признак вывода формулы на экран
  my $side = shift;
  my $formula = shift;

  my $len;

  # периметр равен: 4 * длина стороны
  $len = 4 * $side;

  if ( $formula )
  {
    print "Формула для расчета периметра квадрата: \n" .
          "--- длина стороны * 4 ---\n";
  }

  return ( $len );
}


1;  # это возвращаемое значение "истина" (должно присутствовать обязательно)
