/*
 * : test.c
 *          libpq
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo; /*       */
  PGconn     *conn;     /*      */
  PGresult   *res;      /*   SQL- */
  int        nFields;   /*     */
  int        i, j;

  /*
   *       ,
   *        conninfo;
   *       ,   
   *       
   * "dbname=test user=postgres"
  */
  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /*     */
  conn = PQconnectdb( conninfo );

  /*    */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "     : %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /*  PQexec  SQL-,  
         */
  res = PQexec( conn, "SELECT * FROM Students" );
  if (PQresultStatus(res) != PGRES_TUPLES_OK)
  {
    fprintf( stderr, " SELECT : %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /*     */
  
  /*        
     (     Students) */
  for ( j = 0; j < nFields; j++ )
  {
    /*   ", , "   
       ( PQfname      
         ) */
    if ( strcmp( PQfname( res, j ), "name" ) == 0 )
      printf( "%-30s", PQfname( res, j ) );
    else
      printf( "%-10s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /*     
     ( PQntuples     ) */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "name" ) == 0 )
        /*  PQgetvalue     i
              j */
        printf( "%-30s", PQgetvalue( res, i, j ) );
      else
        printf( "%-10s", PQgetvalue( res, i, j ) );
    }
    printf( "\n" );
  }

  /*  ,    res,
            */
  PQclear( res );

  /*      */
  PQfinish( conn );

  return 0;
}
