/*
 * : test1.c
 *         
 *    .
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo; /*       */
  PGconn     *conn;     /*      */
  PGresult   *res;      /*   SQL- */
  int        nFields;   /*     */
  int        i, j;

  char command[ 256 ];    /*    SQL- */
  char record_book[ 10 ]; /*       */

  /*
   *       ,
   *        conninfo;
   *       ,   
   *       
   * "dbname=test user=postgres"
  */
  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /*    . */
  conn = PQconnectdb( conninfo );

  /*   . */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "     : %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /*      . */
  printf( "   : " );
  scanf( "%s", record_book );

  /*   SQL-. */
  sprintf( command, "SELECT * FROM students WHERE record_book = %s",
           record_book );

  /*  PQexec  SQL-,  
         */
  res = PQexec( conn, command );

  if ( PQresultStatus( res ) != PGRES_TUPLES_OK )
  {
    fprintf( stderr, " SELECT : %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /*     */
  
  /*        
     (     students). */
  for ( j = 0; j < nFields; j++ )
  {
    /*   ", , "   
       ( PQfname      
         ). */
    if ( strcmp( PQfname( res, j ), "name" ) == 0 )
      printf( "%-30s", PQfname( res, j ) );
    else
      printf( "%-12s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /*     
     ( PQntuples     ). */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "name" ) == 0 )
        /*  PQgetvalue     j
              i. */
        printf( "%-30s", PQgetvalue( res, i,j ) );
      else
        printf( "%-12s", PQgetvalue( res, i,j ) );
    }
    printf( "\n" );
  }

  /*  ,    res,
           . */
  PQclear( res );

  /* ----------------------------- */

  /*   SQL-. */
  sprintf( command, "SELECT * FROM progress WHERE record_book = %s",
           record_book );

  /*  PQexec  SQL-,  
        . */
  res = PQexec( conn, command );

  if ( PQresultStatus( res ) != PGRES_TUPLES_OK )
  {
    fprintf( stderr, " SELECT : %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /*     */
  
  /*        
     (     progress). */
  for ( j = 0; j < nFields; j++ )
  {
    /*   ", , "   
       ( PQfname      
         ). */
    if ( strcmp( PQfname( res, j ), "record_book" ) == 0 )
      printf( "%-12s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "acad_year" ) == 0 )
      printf( "%-10s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "term" ) == 0 )
      printf( "%-5s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "discipline_code" ) == 0 )
      printf( "%-16s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "mark" ) == 0 )
      printf( "%-5s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /*     
     ( PQntuples     ). */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "record_book" ) == 0 )
        /*  PQgetvalue     i
              j. */
        printf( "%-12s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "acad_year" ) == 0 )
        printf( "%-10s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "term" ) == 0 )
        printf( "%-5s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "discipline_code" ) == 0 )
        printf( "%-16s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "mark" ) == 0 )
        printf( "%-5s", PQgetvalue( res, i, j ) );
    }
    printf( "\n" );
  }

  /*  ,    res,
           . */
  PQclear( res );

  /*     . */
  PQfinish( conn );

  return 0;
}
