/*
 * Программа: test1.c
 * Запрашиваем номер зачетной книжки и отображаем данные о соответствующем
 * студенте из двух таблиц.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo; /* параметры для соединения с базой данных */
  PGconn     *conn;     /* дескриптор соединения с базой данных */
  PGresult   *res;      /* результат выполнения SQL-запроса */
  int        nFields;   /* число полей в выборке */
  int        i, j;

  char command[ 256 ];    /* буфер для формирования SQL-команды */
  char record_book[ 10 ]; /* буфер для записи номера зачетной книжки */

  /*
   * Если пользователь передает параметр в командной строке,
   * то нужно использовать его в качестве строки conninfo;
   * если параметр в командной строке не передан, то по умолчанию
   * для подключения к базе данных используется строка
   * "dbname=test user=postgres"
  */
  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /* Подключаемся к базе данных. */
  conn = PQconnectdb( conninfo );

  /* Проверяем успешность подключения. */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "Подключиться к базе данных не удалось: %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /* Запросим у пользователя номер зачетной книжки. */
  printf( "Введите номер зачетной книжки: " );
  scanf( "%s", record_book );

  /* Формируем строку SQL-команды. */
  sprintf( command, "SELECT * FROM students WHERE record_book = %s",
           record_book );

  /* функция PQexec исполняет SQL-запрос, переданный ей
     в виде символьной строки */
  res = PQexec( conn, command );

  if ( PQresultStatus( res ) != PGRES_TUPLES_OK )
  {
    fprintf( stderr, "Запрос SELECT неудачен: %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /* количество полей в выборке */
  
  /* Сначала выведем имена полей для полученной выборки 
     (фактически это имена полей таблицы students). */
  for ( j = 0; j < nFields; j++ )
  {
    /* Для поля "Фамилия, имя, отчество" выделим больше места
       (функция PQfname возвращает имя поля по его порядковому
       номеру в выборке). */
    if ( strcmp( PQfname( res, j ), "name" ) == 0 )
      printf( "%-30s", PQfname( res, j ) );
    else
      printf( "%-12s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /* Выведем данные из полученной выборки
     (функция PQntuples возвращает число строк в выборке). */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "name" ) == 0 )
        /* Функция PQgetvalue возвращает значение поля номер j
           из строки номер i. */
        printf( "%-30s", PQgetvalue( res, i,j ) );
      else
        printf( "%-12s", PQgetvalue( res, i,j ) );
    }
    printf( "\n" );
  }

  /* Освободим память, которую занимала структура res,
     как только эта структура больше не нужна. */
  PQclear( res );

  /* ----------------------------- */

  /* Формируем строку SQL-команды. */
  sprintf( command, "SELECT * FROM progress WHERE record_book = %s",
           record_book );

  /* Функция PQexec исполняет SQL-запрос, переданный ей
     в виде символьной строки. */
  res = PQexec( conn, command );

  if ( PQresultStatus( res ) != PGRES_TUPLES_OK )
  {
    fprintf( stderr, "Запрос SELECT неудачен: %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }

  nFields = PQnfields( res );  /* количество полей в выборке */
  
  /* Сначала выведем имена полей для полученной выборки 
     (фактически это имена полей таблицы progress). */
  for ( j = 0; j < nFields; j++ )
  {
    /* Для поля "Фамилия, имя, отчество" выделим больше места
       (функция PQfname возвращает имя поля по его порядковому
       номеру в выборке). */
    if ( strcmp( PQfname( res, j ), "record_book" ) == 0 )
      printf( "%-12s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "acad_year" ) == 0 )
      printf( "%-10s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "term" ) == 0 )
      printf( "%-5s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "discipline_code" ) == 0 )
      printf( "%-16s", PQfname( res, j ) );
    else if ( strcmp( PQfname( res, j ), "mark" ) == 0 )
      printf( "%-5s", PQfname( res, j ) );
  }

  printf( "\n\n" );

  /* Выведем данные из полученной выборки
     (функция PQntuples возвращает число строк в выборке). */
  for ( i = 0; i < PQntuples( res ); i++ )
  {
    for ( j = 0; j < nFields; j++ )
    {
      if ( strcmp( PQfname( res, j ), "record_book" ) == 0 )
        /* Функция PQgetvalue возвращает значение поля номер i
           из строки номер j. */
        printf( "%-12s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "acad_year" ) == 0 )
        printf( "%-10s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "term" ) == 0 )
        printf( "%-5s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "discipline_code" ) == 0 )
        printf( "%-16s", PQgetvalue( res, i, j ) );
      else if ( strcmp( PQfname( res, j ), "mark" ) == 0 )
        printf( "%-5s", PQgetvalue( res, i, j ) );
    }
    printf( "\n" );
  }

  /* Освободим память, которую занимала структура res,
     как только эта структура больше не нужна. */
  PQclear( res );

  /* Закроем соединение с базой данных. */
  PQfinish( conn );

  return 0;
}
