/*
 * Программа: test3.c
 * Вставка в БД новой записи, данные запрашиваются с клавиатуры.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libpq-fe.h"

static void exit_nicely( PGconn *conn )
{
  PQfinish( conn );
  exit( 1 );
}

int main( int argc, char **argv )
{
  const char *conninfo;   /* параметры для соединения с базой данных */
  PGconn     *conn;       /* дескриптор соединения с базой данных */
  PGresult   *res;        /* результат выполнения SQL-запроса */
  int        nFields;     /* число полей в выборке */
  int        i, j;

  char command[ 512 ];    /* буфер для формирования SQL-команды, */
  char record_book[ 6 ];  /* буфер для записи номера зачетной книжки */
  char name[ 50 ];        /* буфер для записи Ф.И.О. */
  char psp_ser[ 5 ];      /* буфер для записи серии паспорта */
  char psp_num[ 7 ];      /* буфер для записи номера паспорта  */

  /* Необходимо сформировать символьную строку SQL-команды */

  printf( "Введите номер зачетной книжки: " );
  scanf( "%s", record_book );    

  /* Очистим буфер ввода, в котором остался символ новой строки '\n'. */
  scanf( "%*[^\n]" );  /* Удаляем все символы вплоть до символа '\n'. */
  scanf( "%*c" );      /* Удаляем символ '\n'. */

  printf( "Введите Ф.И.О.: " );
  fgets( name, 49, stdin );

  /* Так как функция fgets() записывает в буфер и символ '\n', введенный
     пользователем при нажатии клавиши Enter, то запишем на его место
     символ '\0', означающий конец строки. */
  name[ strlen( name ) - 1 ] = '\0';

  printf( "Введите серию паспорта: " );
  scanf( "%s",  psp_ser );
  printf( "Введите номер паспорта: " );
  scanf( "%s",  psp_num );

  /* Формируем строку SQL-команды. */

  sprintf( command,
          "INSERT INTO students ( record_book, name, psp_ser, psp_num ) "\
          "VALUES ( %s, '%s', %s, %s )",
          record_book, name, psp_ser, psp_num );

  if ( argc > 1 )
    conninfo = argv[ 1 ];
  else
    conninfo = "dbname=test user=postgres";

  /* Подключаемся к базе данных. */
  conn = PQconnectdb( conninfo );

  /* Проверяем успешность подключения. */
  if ( PQstatus( conn ) != CONNECTION_OK )
  {
    fprintf( stderr, "Подключиться к базе данных не удалось: %s",
             PQerrorMessage( conn ) );
    exit_nicely( conn );
  }

  /* Функция PQexec исполняет SQL-запрос, переданный ей
     в виде символьной строки. */
  res = PQexec( conn, command );

  /* Эта константа -- PGRES_COMMAND_OK -- для команд, модифицирующих таблицы,
     а не выбирающих записи, т. е. INSERT, UPDATE, DELETE. */
  if ( PQresultStatus( res ) != PGRES_COMMAND_OK )
  {
    fprintf( stderr, "Команда INSERT неудачна: %s",
             PQerrorMessage( conn ) );
    PQclear( res );
    exit_nicely( conn );
  }
  else
  {
    printf("Запись вставлена в таблицу students."); 
  }

  /* Освободим память, которую занимала структура res,
     как только эта структура больше не нужна. */
  PQclear( res );

  /* Закроем соединение с базой данных. */
  PQfinish( conn );

  return 0;
}
