<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  отображение информации об учебных дисциплинах на экране
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

/*
  Отображение формы для ввода новой записи или корректировки существующей
  записи в таблице disciplines.
*/
function display_discipline_form( $discipline_info )
{
  // Параметр: $discipline_info -- массив данных об одной дисциплине.
  // Этот параметр может и не быть передан, если форма используется
  // для ввода новой записи, а не для корректировки существующей.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"disciplines.js\"></script>\n";

  // Если операция -- обновление записи, тогда эта функция получила
  // параметр $discipline_info.
  if ( isset( $discipline_info ) )
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=update";
    $operation = 'update_discipline';
    $discipline_code = $discipline_info[ 'discipline_code' ];
    $discipline_name = $discipline_info[ 'discipline_name' ];
  }
  else  // добавление новой записи
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=insert";
    $operation = 'add_discipline';
    $discipline_code = '';
    $discipline_name = '';
  }

  // Атрибут action содержит имя скрипта, который получает набор значений,
  // введенных в поля формы. Событие onsubmit вызывает функцию для проверки
  // значений полей формы.
  // Скрытое поле (type="hidden") с именем name="current_discipline_code"
  // нужно для того, чтобы при изменении значения поля "Код учебной
  // дисциплины" мы могли найти в таблице запись со старым значением
  // этого поля.
  echo <<<END
<form id="discipline_form" action="$script_name" method="post"
      onsubmit="return check_discipline_form()">
  <fieldset id="disciplines">
    <legend>Учебная дисциплина</legend>
    <label for="d_code">Код учебной дисциплины</label>
    <input id="d_code" type="text" name="discipline_code" size="4"
     maxlength="3" value="$discipline_code"
     title="Число цифр в коде дисциплины от 1 до 3" />

    <input type="hidden" name="current_discipline_code"
     value="$discipline_code" />

    <label for="d_name">Наименование учебной дисциплины</label>
    <input id="d_name" type="text" name="discipline_name" size="60"
     maxlength="100" value="$discipline_name" />

    <input class="submit" type="submit" name="$operation" value="Записать" />

    <p id="error"></p>
  </fieldset>
</form>

END;

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">" .
       "Показать список дисциплин</a><br />\n";

  include 'foot.php';
}

/*
  Вывод всех записей из таблицы disciplines.
*/
function display_disciplines_list( $disciplines_list, $select_flag )
{
  // Параметры: $disciplines_list -- двухмерный массив данных о дисциплинах;
  //            $select_flag -- признак вызова этой функции в режиме
  //                            ВЫБОРА учебной дисциплины из списка.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"disciplines.js\"></script>\n";
  echo "<script src=\"university.js\"></script>\n";

  echo "<h2>Учебные дисциплины</h2>\n";

  /* В режиме выбора дисциплин добавление записей не разрешим. */
  if ( ! isset( $select_flag ) )
    echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
         "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  // Выведем результаты в виде таблицы.
  echo "<table id=\"disciplines_table\">\n";

  // Заголовок таблицы.
  echo "  <tr>\n";
  echo "    <th>Код дисциплины</th>\n" ;
  echo "    <th>Наименование дисциплины</th>\n" ;

  if ( isset( $select_flag ) )
    echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
  else
  {
    echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
    echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
  }
  echo "  </tr>\n";

  // Будем формировать уникальные идентификаторы всех строк в таблице:
  // row0, row1 и т. д.
  $row = 0;

  // Берем из двухмерного массива строку за строкой. Каждая строка
  // является хеш-массивом, поэтому возможно обращение типа:
  // $disc[ 'discipline_name' ].
  foreach ( $disciplines_list as $disc )
  {
    echo "<tr id=\"row" . $row . "\">\n";

    echo "<td class=\"center\">" . $disc[ 'discipline_code' ] . "</td>\n";
    echo "<td>" . $disc[ 'discipline_name' ] . "</td>\n";

    // Если этот модуль вызван в режиме выбора учебной дисциплины из списка...
    if ( isset( $select_flag ) )
    {
      echo "<td><a href=\"javascipt:void( 0 )\" " .
           "onclick=\"assign_values( " . $disc[ 'discipline_code' ] .
           ", '" . $disc[ 'discipline_name' ] . "' )\">Выбрать</a></td>\n";
    }
    else
    {
      // Конструкция javascript:void( 0 ) предназначена для того, чтобы
      // при выборе этой ссылки только вызывались функции, связанные
      // с событием onclick, а сам скрипт повторно не запускался.
      // Выражение $_SERVER[ 'SCRIPT_NAME' ] означает имя скрипта, который
      // и сформировал эту HTML-страницу. Он вызывается повторно для удаления
      // текущей записи из таблицы. При этом скрипту передаются параметры
      // методом GET. Они идут парами: имя параметра и значение параметра.
      // Пары разделяются знаком "&".
      echo "<td><a href=\"javascript:void( 0 )\" " .
           "onclick=\"confirm( 'Удалить запись?' ) && delete_discipline( '" .
           $_SERVER[ 'SCRIPT_NAME' ] . "', " .  $disc[ 'discipline_code' ] .
           ", 'row" . $row . "' )\"" .
           " target=\"_self\">Удалить</a></td>\n";

      echo "<td><a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
           "?operation=update&discipline_code=" . $disc[ 'discipline_code' ] .
           "\" target=\"_self\">Изменить</a></td>\n";
    }
    
    echo "</tr>\n";

    $row++;
  }

  echo "</table>\n";

  include 'foot.php';  
  return 0;
}

/*
  Сообщение о добавлении новой записи в таблицу disciplines.
*/
function inform_inserted_discipline( $rec_num )
{
  // Параметр: $rec_num -- число добавленных записей (1 или 0).

  // В этом файле содержится информация, на основе которой формируется
  // верхняя часть HTML-страницы.
  include 'head.php';

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись добавлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не добавлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">" .
       "Показать список дисциплин</a><br />\n";

  // В этом файле содержится информация, на основе которой формируется
  // нижняя часть HTML-страницы.
  include 'foot.php';
}

/*
  Сообщение об обновлении записи в таблице disciplines.
*/
function inform_updated_discipline( $rec_num )
{
  // Параметр: $rec_num -- число обновленных записей (1 или 0).

  include 'head.php';

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись обновлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не обновлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">" .
       "Показать список дисциплин</a><br />\n";

  include 'foot.php';
}

?>
