<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  "Экзаменационные оценки" (отображение информации)
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

/*
  Отображение формы для ввода новой записи или корректировки существующей
  записи в таблице progress.
*/
function display_exam_form( $record_book, $name, $exam_info )
{
  // Параметры: $record_book -- номер зачетной книжки;
  //            $name -- Ф. И. О. студента;
  //            $exam_info -- массив данных об одном экзамене.
  //            Этот параметр может и не быть передан, если форма
  //            используется для ввода новой записи, а не для 
  //            корректировки существующей.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"progress.js\"></script>\n";

  // Если операция -- обновление записи, тогда эта функция получила
  // параметр $exam_info.
  if ( isset( $exam_info ) )
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=update";
    $operation = 'update_exam';
    $acad_year = $exam_info[ 'acad_year' ];
    $term = $exam_info[ 'term' ];
    $discipline_code = $exam_info[ 'discipline_code' ];
    $discipline_name = $exam_info[ 'discipline_name' ];
    $mark = $exam_info[ 'mark' ];
  }
  else  // добавление новой записи
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=insert";
    $operation = 'add_exam';
    $acad_year = '2017/2018'; // по умолчанию
    $term = '1';              // по умолчанию
    $discipline_code = '';
    $discipline_name = '';
    $mark = '5';              // по умолчанию
  }

  // Атрибут action содержит имя скрипта, который получает набор значений,
  // введенных в поля формы. Событие onsubmit вызывает функцию для проверки
  // значений полей формы.
  // Скрытые поля (type="hidden") с именами name="current_record_book" и др.
  // нужны для того, чтобы при изменении значения поля "Номер зачетной
  // книжки", "Учебный год" и др. мы могли найти в таблице запись со старыми
  // значениями этих полей.
  // Скрытое поле (type="hidden") с именем name="name" -- это Ф. И. О.
  // студента. Его нужно передать дальше, чтобы сформировать ссылку
  // в функции inform_inserted_exam.
  // Скрытые поля также передаются для обработки при нажатии кнопки Submit.
  echo <<<END
<form id="exam_form" action="$script_name" method="post"
      onsubmit="return check_exam_form()">
  <fieldset id="personal">
    <legend>Экзаменационные оценки&nbsp;&nbsp;$name </legend>
    <label for="r_book">Номер зачетной книжки</label>
    <input id="r_book" type="text" name="record_book" size="6"
     maxlength="5" value="$record_book" readonly="readonly" />

    <input type="hidden" name="current_record_book" value="$record_book" />
    <input type="hidden" name="current_acad_year" value="$acad_year" />
    <input type="hidden" name="current_term" value="$term" />
    <input type="hidden" name="current_discipline_code"
     value="$discipline_code" />

    <label for="a_year">Учебный год</label>
    <select id="a_year" name="acad_year"> 
END;
  foreach ( array( '2017/2018', '2016/2017', '2015/2016' ) as $ay )
  {
    echo "      <option value=\"$ay\"";
    if ( $acad_year == $ay )
      echo " selected=\"selected\"";

    echo ">$ay</option>\n";
  }
  echo "   </select>\n";

  echo "   <label for=\"term\">Семестр</label>\n";

  foreach ( array( '1', '2' ) as $t )
  {

    echo "      <input type=\"radio\" id=\"term\" name=\"term\" value=\"$t\"";
    if ( $term == $t )
      echo " checked=\"checked\"";

    echo "> $t\n";
  }

  // В этом фрагменте используется атрибут style тега <input> для того,
  // чтобы вывести ссылку на одной строке с полем для ввода кода
  // дисциплины.
  echo <<<END
    <label for="d_code">Код дисциплины</label>
    <input id="d_code" type="text" name="discipline_code" size="3"
     maxlength="4" value="$discipline_code" style="display: inline" />
    <a href="javascript:void( 0 )" onclick="select_discipline()">
     Список дисциплин</a>

    <label for="d_name">Наименование дисциплины</label>
    <textarea id="d_name" name="discipline_name" rows="3" cols="60"
     readonly="readonly"> $discipline_name </textarea>

    <label for="mark">Оценка</label>
    <select id="mark" name="mark"> 
END;

  foreach ( array( 5 => 'отлично',
                   4 => 'хорошо',
                   3 =>  'удовлетворительно' ) as $digit => $word )
  {
    echo "      <option value=\"$digit\"";
    if ( $mark == $digit )
      echo " selected=\"selected\"";

    echo ">$word</option>\n";
  }
  echo <<<END
    </select>

    <input type="hidden" name="name" value="$name" />
    <input class="submit" type="submit" name="$operation" value="Записать" />

    <p id="error"></p>
  </fieldset>
</form>
END;

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list&record_book=$record_book&name=$name\" " .
       "target=\"_self\">Показать все оценки</a><br />\n";

  include 'foot.php';
}

/*
  Вывод всех записей из таблицы progress для указанного номера зачетной
  книжки.
*/
function display_exams_list( $record_book, $name, $exams_list )
{
  // Параметры: $record_book -- номер зачетной книжки;
  //            $name -- Ф. И. О. студента;
  //            $exams_list -- двухмерный массив данных об успеваемости
  //                           одного студента.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"progress.js\"></script>\n";
  echo "<script src=\"university.js\"></script>\n";

  echo "<h2>Результаты экзаменационных сессий</h2>\n";

  echo "<h3>$record_book $name</h3>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert&record_book=$record_book&name=$name\" " .
       "target=\"_self\">Добавить запись</a><br />\n";

  // Выведем результаты в виде таблицы.
  echo "<table id=\"exams_table\">\n";

  // Заголовок таблицы.
  echo "<tr>\n";
  // echo "<th>Номер зачетной книжки</th>\n" ;
  echo "<th>Учебный год</th>\n" ;
  echo "<th>Семестр</th>\n" ;
  echo "<th>Дисциплина</th>\n" ;
  echo "<th>Оценка</th>\n" ;
  echo "<th>&nbsp;</th>\n" ;  // пустая ячейка
  echo "<th>&nbsp;</th>\n" ;  // пустая ячейка
  echo "</tr>\n";

  // Будем формировать уникальные идентификаторы всех строк в таблице:
  // row0, row1 и т. д.
  $row = 0;

  // Берем из двухмерного массива строку за строкой. Каждая строка
  // является хеш-массивом, поэтому возможно обращение типа: $stud[ 'name' ].
  foreach ( $exams_list as $exam )
  {
    echo "<tr id=\"row" . $row . "\">\n";

    echo "<td class=\"center\">" . $exam[ 'acad_year' ] . "</td>\n";
    echo "<td class=\"center\">" . $exam[ 'term' ] . "</td>\n";
    echo "<td>" . $exam[ 'discipline_name' ] . "</td>\n";
    echo "<td class=\"center\">" . $exam[ 'mark' ] . "</td>\n";

    // Конструкция javascript:void( 0 ) предназначена для того, чтобы
    // при выборе этой ссылки только вызывались функции, связанные
    // с событием onclick, а сам скрипт повторно не запускался.
    // Выражение $_SERVER[ 'SCRIPT_NAME' ] означает имя скрипта, который
    // и сформировал эту HTML-страницу. Он вызывается повторно для удаления
    // текущей записи из таблицы. При этом скрипту передаются параметры
    // методом GET. Они идут парами: имя параметра и значение параметра.
    // Пары разделяются знаком "&".
    echo "<td><a href=\"javascript:void( 0 );\" " .
         "onclick=\"confirm( 'Удалить запись?' ) && delete_exam( '" .
         $_SERVER[ 'SCRIPT_NAME' ] . "', " . $exam[ 'record_book' ] .
         ", '" . $exam[ 'acad_year' ] . "', " .  $exam[ 'term' ] .
         ", " . $exam[ 'discipline_code' ] . ", 'row" . $row . "' )\"" .
         " target=\"_self\">Удалить</a></td>\n";

    // Два параметра: наименование операции (update) и номер зачетной книжки.
    echo "<td><a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
         "?operation=update&record_book=" . $exam[ 'record_book' ] .
         "&acad_year=" . $exam[ 'acad_year' ] . "&term=" .  $exam[ 'term' ] .
         "&discipline_code=" . $exam[ 'discipline_code' ] .  "&name=$name" .
         "\" target=\"_self\">Изменить</a></td>\n";

    echo "</tr>\n";

    $row++;
  }

  echo "</table>\n";

  include 'foot.php';  
  return 0;
}

/*
  Сообщение о добавлении новой записи в таблицу progress.
*/
function inform_inserted_exam( $rec_num )
{
  // Параметр: $rec_num -- число добавленных записей (1 или 0).

  // В этом файле содержится информация, на основе которой формируется
  // верхняя часть HTML-страницы.
  include 'head.php';

  echo "<h3>" . $_POST[ 'record_book' ] . " " . $_POST[ 'name' ] . "</h3>\n";

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись добавлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не добавлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert&record_book=" . $_POST[ 'record_book' ] .
       "&name=" . $_POST[ 'name' ] . "\" " .
       "target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list&record_book=" . $_POST[ 'record_book' ] .
       "&name=" . $_POST[ 'name' ] . "\" " .
       "target=\"_self\">Показать все оценки</a><br />\n";

  // В этом файле содержится информация, на основе которой формируется
  // нижняя часть HTML-страницы.
  include 'foot.php';
}

/*
  Сообщение об обновлении записи в таблице progress.
*/
function inform_updated_exam( $rec_num )
{
  // Параметр: $rec_num -- число обновленных записей (1 или 0).

  include 'head.php';

  echo "<h3>" . $_POST[ 'record_book' ] . " " . $_POST[ 'name' ] . "</h3>\n";

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись обновлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не обновлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert&record_book=" . $_POST[ 'record_book' ] .
       "&name=" . $_POST[ 'name' ] . "\" " .
       "target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list&record_book=" . $_POST[ 'record_book' ] .
       "&name=" . $_POST[ 'name' ] . "\" " .
       "target=\"_self\">Показать все оценки</a><br />\n";

  include 'foot.php';
}

?>
