<?php
/*
  Система: "Учет успеваемости студентов"
  Модуль:  отображение персональной информации о студентах на экране
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

/*
  Отображение формы для ввода новой записи или корректировки существующей
  записи в таблице students.
*/
function display_student_form( $student_info )
{
  // Параметр: $student_info -- массив данных об одном студенте.
  // Этот параметр может и не быть передан, если форма используется
  // для ввода новой записи, а не для корректировки существующей.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"students.js\"></script>\n";

  // Если операция -- обновление записи, тогда эта функция получила
  // параметр $student_info.
  if ( isset( $student_info ) )
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=update";
    $operation = 'update_student';
    $record_book = $student_info[ 'record_book' ];
    $name = $student_info[ 'name' ];
    $psp_ser = $student_info[ 'psp_ser' ];
    $psp_num = $student_info[ 'psp_num' ];
  }
  else  // добавление новой записи
  {
    $script_name = $_SERVER[ 'SCRIPT_NAME' ] . "?operation=insert";
    $operation = 'add_student';
    $record_book = '';
    $name = '';
    $psp_ser = '';
    $psp_num = '';
  }

  // Атрибут action содержит имя скрипта, который получает набор значений,
  // введенных в поля формы. Событие onsubmit вызывает функцию для проверки
  // значений полей формы.
  // Скрытое поле (type="hidden") с именем name="current_record_book"
  // нужно для того, чтобы при изменении значения поля "Номер зачетной
  // книжки" мы могли найти в таблице запись со старым значением этого поля.
  echo <<<END
<form id="student_form" action="$script_name" method="post"
      onsubmit="return check_student_form()">
  <fieldset id="personal">
    <legend>Персональные данные</legend>
    <label for="r_book">Номер зачетной книжки</label>
    <input id="r_book" type="text" name="record_book" size="6"
     maxlength="5" value="$record_book" title="Число цифр в номере равно 5" />

    <input type="hidden" name="current_record_book" value="$record_book" />

    <label for="f_name">Фамилия, имя, отчество</label>
    <input id="f_name" type="text" name="name" size="40" maxlength="50"
     value="$name" />

    <label for="p_ser">Серия паспорта</label>
    <input id="p_ser" type="text" name="psp_ser" size="5" maxlength="4"
     value="$psp_ser" title="Число цифр в серии паспорта равно 4" />

    <label for="p_num">Номер паспорта</label>
    <input id="p_num" type="text" name="psp_num" size="7" maxlength="6"
     value="$psp_num" title="Число цифр в номере паспорта равно 6" />

    <input class="submit" type="submit" name="$operation" value="Записать" />

    <p id="error"></p>
  </fieldset>
</form>

END;

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">Показать список студентов</a>\n";

  include 'foot.php';
}

/*
  Вывод всех записей из таблицы students.
*/
function display_students_list( $students_list )
{
  // Параметр: $students_list -- двухмерный массив данных о студентах.

  include 'head.php';

  // Таким образом можно подключить файл, содержащий код функций
  // на языке JavaScript.
  echo "<script src=\"students.js\"></script>\n";
  echo "<script src=\"university.js\"></script>\n";

  echo "<h2>Персональные данные студентов</h2>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  // Выведем результаты в виде таблицы.
  echo "<table id=\"students_table\">\n";

  // Заголовок таблицы.
  echo "  <tr>\n";
  echo "    <th>Номер зачетной книжки</th>\n" ;
  echo "    <th>Фамилия, имя, отчество</th>\n" ;
  echo "    <th>Серия паспорта</th>\n" ;
  echo "    <th>Номер паспорта</th>\n" ;
  echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
  echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
  echo "    <th>&nbsp;</th>\n" ;  // пустая ячейка
  echo "  </tr>\n";

  // Будем формировать уникальные идентификаторы всех строк в таблице:
  // row0, row1 и т. д.
  $row = 0;

  // Берем из двухмерного массива строку за строкой. Каждая строка
  // является хеш-массивом, поэтому возможно обращение типа: $stud[ 'name' ].
  foreach ( $students_list as $stud )
  {
    echo "<tr id=\"row" . $row . "\">\n";

    echo "<td class=\"center\">" . $stud[ 'record_book' ] . "</td>\n";
    echo "<td>" . $stud[ 'name' ] . "</td>\n";
    echo "<td class=\"center\">" . $stud[ 'psp_ser' ] . "</td>\n";
    echo "<td class=\"center\">" . $stud[ 'psp_num' ] . "</td>\n";

    // Конструкция javascript:void( 0 ) предназначена для того, чтобы
    // при выборе этой ссылки только вызывались функции, связанные
    // с событием onclick, а сам скрипт повторно не запускался.
    // Выражение $_SERVER[ 'SCRIPT_NAME' ] означает имя скрипта, который
    // и сформировал эту HTML-страницу. Он вызывается повторно для удаления
    // текущей записи из таблицы. При этом скрипту передаются параметры
    // методом GET. Они идут парами: имя параметра и значение параметра.
    // Пары разделяются знаком "&".
    echo "<td><a href=\"javascript:void( 0 );\" " .
         "onclick=\"confirm( 'Удалить запись?' ) && delete_student( '" .
         $_SERVER[ 'SCRIPT_NAME' ] . "', " .  $stud[ 'record_book' ] .
         ", 'row" . $row . "' )\"" .
         " target=\"_self\">Удалить</a></td>\n";

    // Два параметра: наименование операции (update) и номер зачетной книжки.
    echo "<td><a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
         "?operation=update&record_book=" . $stud[ 'record_book' ] .
         "\" target=\"_self\">Изменить</a></td>\n";

    // Два параметра: наименование операции (progress) и номер зачетной книжки.
    echo "<td><a href=\"progress.php" . 
         "?operation=progress&record_book=" . $stud[ 'record_book' ] .
         "&name=" . $stud[ 'name' ] .
         "\" target=\"_blank\">Экзаменационные оценки</a></td>\n";

    echo "</tr>\n";

    $row++;
  }

  echo "</table>\n";

  include 'foot.php';  
  return 0;
}

/*
  Сообщение о добавлении новой записи в таблицу students.
*/
function inform_inserted_student( $rec_num )
{
  // Параметр: $rec_num -- число добавленных записей (1 или 0).

  // В этом файле содержится информация, на основе которой формируется
  // верхняя часть HTML-страницы.
  include 'head.php';

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись добавлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не добавлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">Показать список студентов</a><br />\n";

  // В этом файле содержится информация, на основе которой формируется
  // нижняя часть HTML-страницы.
  include 'foot.php';
}

/*
  Сообщение об обновлении записи в таблице students.
*/
function inform_updated_student( $rec_num )
{
  // Параметр: $rec_num -- число обновленных записей (1 или 0).

  include 'head.php';

  if ( $rec_num == 1 )
    echo "<p class=\"messages\">Запись обновлена</p>\n";
  elseif ( $rec_num == 0 )
    echo "<p class=\"messages\">Запись не обновлена</p>\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=insert\" target=\"_self\">Добавить запись</a><br />\n";

  echo "<a href=\"" . $_SERVER[ 'SCRIPT_NAME' ] . 
       "?operation=list\" target=\"_self\">Показать список студентов</a><br />\n";

  include 'foot.php';
}

?>
