/*
  Система: "Учет успеваемости студентов"
  Модуль:  функции на языке JavaScript, используемые в разных модулях
  Автор:   Е. П. Моргунов
  Дата:    30.11.2017
  Версия:  0.1
*/

/*
  Выполнение удаления одной записи из таблицы.
  Здесь используется технология AJAX, позволяющая получить данные 
  из базы данных и на их основе обновить лишь часть HTML-страницы,
  не переформируя ее заново всю полностью.
*/
function delete_record( script_name, script_params, row )
{
  // Параметры: script_name -- имя скрипта, который будет выполнять
  //                           удаление записи из таблицы;
  //            script_params -- строковая переменная, содержащая параметры
  //                             скрипта;
  //            row -- идентификатор строки в таблице на экране.
  
  // Создаем стандартный объект XMLHttpRequest.
  var xhttp = new XMLHttpRequest();

  // Сформируем функцию-обработчик для события onreadystatechange.
  // Эта функция вызывается не в момент ее создания, а при изменении 
  // состояния объекта XMLHttpRequest, т. е. при отправке запроса
  // web-серверу и при получении ответа от него.
  xhttp.onreadystatechange = function()
  {
    // Если получен ответ от сервера и его статус -- 200 (OK)...
    if ( this.readyState == 4 && this.status == 200 )
    {
      try
      {
        // Попытаемся разобрать информацию, полученную от web-сервера,
        // и сформировать из нее объект JavaScript. В случае штатной работы
        // она должна быть в формате JSON.
        var server_response = JSON.parse( this.responseText );      
      }
      // Если web-сервер отправил неструктурированную информацию, 
      // то JSON.parse выдаст ошибку. Обработаем эту ситуацию.
      catch ( err )
      {
        // Выведем сообщение, полученное от web-сервера, во всплывающем окне.
        // Это будет сообщение об ошибке, которое возвращает PostgreSQL.
        display_message( this.responseText );

        // DEBUG
        // if ( typeof server_response != "object" )
        // {
        //   alert( 'Не могу разобрать JSON' );
        // }
        return;
      }

      // В функциях delete_discipline (см. файл disciplines_db.php),
      // delete_student (см. файл students_db.php), delete_exam 
      // (см. файл progress_db.php) мы с помощью команды echo выводили 
      // на стандартный вывод JSON-объект, в котором в поле rows содержалось
      // число удаленных записей. Этот JSON-объект в результате попадает
      // от web-сервера в эту процедуру.
      if ( server_response.rows == '1' )
      {
        // Используя иерархию DOM, удалим из таблицы на экране строку
        // с идентификатором, значение которого равно параметру row.
        // Таблица на экране будет автоматически перерисована.
        var table_row = document.getElementById( row );
        // Здесь атрибут parentNode означает родительский объект для строки,
        // т. е. таблицу.
        table_row.parentNode.removeChild( table_row );

        display_message( "<p class=\"messages\">Запись удалена</p>\n" );
      }
      else  // server_response.rows == '0'
      {
        display_message( "<p class=\"messages\">Запись не удалена</p>\n" );
      }
    }
  };

  // Готовим запрос к web-серверу. Параметр "POST" означает название
  // метода, используемого для отправки запроса. Параметр script_name
  // означает имя скрипта, который будет запущен web-сервером для получения
  // ответа на наш запрос. Параметр true означает, что запрос выполняется
  // асинхронно, т. е. наша функция delete_record не ожидает ответа,
  // а "надеется" на созданный ею обработчик событий: когда ответ от 
  // web-сервера придет, тогда он и будет обработан.
  xhttp.open( "POST", script_name, true );
  xhttp.setRequestHeader( "Content-Type",
                          "application/x-www-form-urlencoded" );

  // Непосредственная отправка запроса методом POST. Строка-параметр
  // содержит имена и значения ключевых полей таблицы, из которой
  // удаляется запись.
  xhttp.send( "operation=delete&" + script_params );
}

/*
  Вывод сообщения во всплывающем окне.
*/
function display_message( message )
{
  // Параметры: message -- текст сообщения.

  // Создадим окно. Не будем указывать URL и имя окна, поэтому первые два
  // параметра будут пустыми строками.
  var err_win = window.open( "", "", "menubar=no, status=no, titlebar=no, " +
                             "width=800, height=200" );
  err_win.document.open();  // Начинаем создание документа в новом окне.

  // "Пишем" HTML-теги в созданном окне.
  // Используем символ "\n" -- в конце, а пробелы -- в начале символьных строк,
  // содержащих HTML-теги, чтобы форматирование полученного исходного текста
  // HTML-документа было аккуратным.
  err_win.document.write( "<!DOCTYPE html>\n" );
  err_win.document.write( "<html xmlns=\"http://www.w3.org/1999/xhtml\">\n" );
  err_win.document.write( "<head>\n" );
  err_win.document.write( "  <meta http-equiv=\"Content-Type\" " + 
                          "content=\"text/html; charset=utf-8\" />\n" );
  err_win.document.write( "  <link rel=\"stylesheet\" " +
                          "href=\"university.css\" type=\"text/css\" />\n" );
  err_win.document.write( "</head>\n" );
  err_win.document.write( "<body>\n" );
  err_win.document.write( message ); 
  err_win.document.write( "\n  <div>\n" );
  err_win.document.write( "    <input type=\"button\" value=\"Закрыть\" " +
                          "onclick=\"self.close()\" />\n" );
  err_win.document.write( "  </div>\n" );
  err_win.document.write( "</body>\n" );
  err_win.document.write( "</html>\n" );

  err_win.document.close(); // Завершаем создание документа.
}
